/** @file
  @brief The file contains the list of classes used to manage scren layouts.
*/

#uses "CtrlJsonSerialization"

/**
  @brief The UnHMI_Window class represents a single window
*/

class UnHMI_Window
{
  private int number;
  private string fileName;
  private string panelName;
  private string datapoint;
  private dyn_mixed zoomCoord;

  public UnHMI_Window(int number, string fileName, string panelName, string datapoint, dyn_mixed zoomCoord)
  {
	this.number = number;
	this.fileName = fileName;
	this.panelName= panelName;
	this.datapoint = datapoint;
	this.zoomCoord = zoomCoord;
  }

  public string getFileName()
  {
	return this.fileName;
  }

  public void setFileName(string fileName)
  {
	this.fileName = fileName;
  }

  public string getPanelName()
  {
	return this.panelName;
  }

  public void setPanelName(string panelName)
  {
	this.panelName = panelName;
  }

  public int getNumber()
  {
	return this.number;
  }

  public void setDatapoint(string datapoint)
  {
	this.datapoint = datapoint;
  }

  public string getDatapoint()
  {
	return this.datapoint;
  }

  public void setZoomCoord(string zoomCoord)
  {
	this.zoomCoord = zoomCoord;
  }

  public string getZoomCoord()
  {
	return this.zoomCoord;
  }

};

/**
 @brief The UnHMI_ScreenLayout class represents a single screen layout.
*/

class UnHMI_ScreenLayout
{
  private vector< shared_ptr<UnHMI_Window> > windows;
  private string screenLayoutName;
  private string accessControl;
  private bool splitterMode;
  //------------------------------------------------------------------------------
  /** Default constructor.
  */
  public UnHMI_ScreenLayout(string screenLayoutName)
  {
	this.screenLayoutName = screenLayoutName;
  }

  public string getScreenLayoutName()
  {
	return this.screenLayoutName;
  }

  public string getAccessControl()
  {
	return this.accessControl;
  }

  public void setAccessControl(string accessControl)
  {
	this.accessControl = accessControl;
  }

  public bool getSplitterMode()
  {
	return this.splitterMode;
  }

  public void setSplitterMode(bool splitterMode)
  {
	this.splitterMode = splitterMode;
  }

  public vector< shared_ptr<UnHMI_Window> > getWindows()
  {
	return this.windows;
  }

  public void setWindows(vector< shared_ptr<UnHMI_Window> > windows)
  {
	this.windows = windows;
  }

  public void saveToDatapoint()
  {
	// TODO noSerialize accessControl
	string json = fwJsonSerialize(this);
	string dpName = "UnHMI_ScreenLayout/" + this.screenLayoutName;

	strreplace(dpName ," ", "_");
	if (!dpExists(dpName))
	{
	  dpCreate(dpName, "UnHMI_ScreenLayout");
	}
	dpSet(dpName + ".screenLayout", json);
	dpSet(dpName + ".accessControl", this.accessControl);
	dpSet(dpName + ".splitterMode", this.splitterMode);
  }

  public void loadFromDatapoint()
  {
	string json;
	shared_ptr<UnHMI_ScreenLayout> sl;
	string dpName = "UnHMI_ScreenLayout/" + this.screenLayoutName;
	strreplace(dpName ," ", "_");
	dpGet(dpName + ".screenLayout", json);
	dpGet(dpName + ".accessControl", this.accessControl);
	dpGet(dpName + ".splitterMode", this.splitterMode);
	g_unicosHMI_SplitMode = this.splitterMode;
	fwJsonDeserialize(json, sl, makeMapping("acceptIncompleteJson", true));
	this.windows = sl.getWindows();
  }

  public static shared_ptr<UnHMI_ScreenLayout> createWithDpName(string dpName)
  {
	string json, accessControl;
	bool splitterMode;
	shared_ptr<UnHMI_ScreenLayout> sl;
	dpGet(dpName + ".screenLayout", json);
	dpGet(dpName + ".accessControl", accessControl);
	dpGet(dpName + ".splitterMode", splitterMode);
	fwJsonDeserialize(json, sl, makeMapping("acceptIncompleteJson", true));
	sl.setAccessControl(accessControl);
	sl.setSplitterMode(splitterMode);

	return sl;
  }

  public void deleteDatapoint()
  {
	string dpName = "UnHMI_ScreenLayout/" + this.screenLayoutName;
	strreplace(dpName ," ", "_");
	dpDelete(dpName);
  }

  public shared_ptr<UnHMI_ScreenLayout>  clone(string newScreenLayoutName)
  {
	shared_ptr<UnHMI_ScreenLayout>  ret = new UnHMI_ScreenLayout(newScreenLayoutName);
	ret.setAccessControl(this.getAccessControl());
	ret.setSplitterMode(this.getSplitterMode());
	ret.setWindows(this.getWindows());
	return ret;
  }
};
