/** @file

@brief The library of functions used by unHMI.

*/

#uses "classes/unHMI/UnHMI_ScreenLayout.ctl"
#uses "classes/unHMI/UnHMI_ScreenLayoutHelper.ctl"

const string UNHMI_MAIN_MODULE_NAME = "unicosHMI_1";
//Corresponds to the name of the main panel of unHMI_mainUnicosPanel.pnl
const string UNHMI_MAIN_PANEL_NAME = "mainWindow";
const string UNHMI_DEFAULT_PANEL = "/objects/unHMI/unHMI_defaultPanel.pnl";
const string UNHMI_ZOOMED_MODULE_PREFIX = "zoomed_module";

//Docked Module Names
const dyn_string UNHMI_PREVIEW_MODULE_DOCK_NAMES = makeDynString("zoomNavigatorModule1",
                                                                 "zoomNavigatorModule2",
                                                                 "zoomNavigatorModule3",
                                                                 "zoomNavigatorModule4",
                                                                 "zoomNavigatorModuleDTO");

//Main View Embedded Modules (Defined in unHMI.pnl)
const dyn_string UNHMI_MAIN_VIEW_EMBEDDED_MODULES = makeDynString("MODULE1",
                                                                  "MODULE2",
                                                                  "MODULE3",
                                                                  "MODULE4",
                                                                  "DeviceTreeOverview");
const int UNHMI_MAX_NUMBER_OF_PREVIEW_MODULES = 4;
const int UNHMI_SPLITTER_MODULE_NUMBER = -1;

//Number of Extra Preview Modules needed for automating certain function e.g "setVisible"
// 1-> Dev Overview
const int UNHMI_NUMBER_OF_EXTRA_PREVIEW_MODULES = 1;

const string splitterPanelName = "splitterPanel";
const string UNHMI_ZOOM_NAVIGATOR_PANELNAME = "navigatorPanel";
const string UNHMI_SPLITTER_MODULE_NAME = "splitterModule";
const string UNHMI_SPLITTER_FLAG = "splitter";
//Should also be identical to the name of dockedEmbeddedModule.pnl
const string UNHMI_PREVIEW_PANEL_NAME ="previewPanel";
const string UNHMI_PREVIEW_MODULE_NAME = "previewModule";
const string UNHMI_COMMAND_MODULE_NAME = "commandModule";
//Embedded Panel name of unHMI.pnl
const string UNHMI_SPLITTER_EMBEDDED_PANEL_NAME = "SplitterEmbPanel";
const string UNHMI_TOOLBAR_MODULE_NAME = "unicosToolbarModule";
const string UNHMI_NAVIGATION_BAR_PANELFILE = "objects/unHMI/unHMI_navigationHistoryBar.pnl";
const string EMBEDDED_PANEL_NAME = "panel";

const int UNHMI_REDRAW_REFRESH_RATE = 1;

dyn_string navigationBarDp = makeDynString("navigationBar1","navigationBar2", "navigationBar3", "navigationBar4","navigationBar5");


public void openPreviewModuleOnModule(string previewModuleContainer, string zoomedModuleContainer, string zoomedFileName, string sDpName = "",
                                      dyn_string previewModuleContainerDollars="", dyn_string zoomedModuleContainerDollars="",
                                      string sPanelDollarParams = "") {
  RootPanelOnModule("unHMI/unHMI_mainWindowEmbeddedModule.pnl", EMBEDDED_PANEL_NAME, zoomedModuleContainer, zoomedModuleContainerDollars);

  // If previewModuleContainer already exists no need to re-initialize it
  if (getShape(previewModuleContainer + "." + UNHMI_PREVIEW_PANEL_NAME + ":") == 0) {
    ModuleOnWithPanel(previewModuleContainer,-1,-1,100,100,1,1,"", "unHMI/unHMI_dockedEmbeddedModule.pnl", EMBEDDED_PANEL_NAME, previewModuleContainerDollars);
  }
  invokeMethod(previewModuleContainer + "." + UNHMI_PREVIEW_PANEL_NAME + ":", "unHMI_setPanelDollarParams", sPanelDollarParams);
  invokeMethod(previewModuleContainer + "." + UNHMI_PREVIEW_PANEL_NAME + ":", "unHMI_setZoomedFileName", zoomedFileName, sDpName);
  //Add delay to properly process the modules by the populate action function after setting the preview panel
  delay(1);
  invokeMethod(previewModuleContainer + "." + UNHMI_PREVIEW_PANEL_NAME + ":", "unHMI_populateModulesAction");
}

public void unHMI_openPreviewModuleHelper(string sPanelFileName, int i, string sPanelDollarParams="", string sDpName = "") {
  dyn_string zoomedModuleDollars = makeDynString("$zoomedFileName:"+ sPanelFileName,
                                                 "$zoomedModuleName:"+ UNHMI_ZOOMED_MODULE_PREFIX + i,
                                                 "$title:"+ "Module " + i,
                                                 "$sPanelDollarParams:"+ sPanelDollarParams);
  dyn_string previewDollars = makeDynString("$zoomedModule:" + UNHMI_ZOOMED_MODULE_PREFIX + i,
                                            "$number:" + i,
                                            "$zoomedFileName:" + sPanelFileName);

  bool isSplitterVisible = invokeMethod(UNHMI_MAIN_MODULE_NAME + ".mainWindow:", "unHMI_getVisible", UNHMI_SPLITTER_MODULE_NUMBER);
  //   Different handling if opening on splitter
  if (g_unicosHMI_SplitMode || isSplitterVisible) {
    shape splitterShape = getShape(UNHMI_SPLITTER_MODULE_NAME + "." + splitterPanelName+":");
    int highlighted_splitterModule;
    string sDpPanel, sName;

    if (i == UNHMI_SPLITTER_MODULE_NUMBER) {
      i = splitterShape.unHMI_get_highlighted_splitterModule();
    }
    unHMI_get_sDpPanel_sName(sPanelFileName, sDpPanel, sName);

	if ((i >= 1) && (i <= UNHMI_MAX_NUMBER_OF_PREVIEW_MODULES)) {
      invokeMethod(UNHMI_PREVIEW_MODULE_DOCK_NAMES[i] + "." + UNHMI_PREVIEW_PANEL_NAME + ":", "unHMI_setPanelDollarParams", sPanelDollarParams);
      splitterShape.unHMI_addModule(UNHMI_ZOOMED_MODULE_PREFIX + i, sPanelFileName, i, sName, sDpPanel, sDpName);
	  moduleShow(UNHMI_COMMAND_MODULE_NAME+i);
	  moduleHide(UNHMI_PREVIEW_MODULE_DOCK_NAMES[i]);
	}
  } else {
    openPreviewModuleOnModule(UNHMI_PREVIEW_MODULE_DOCK_NAMES[i], UNHMI_MAIN_VIEW_EMBEDDED_MODULES[i], sPanelFileName, sDpName, previewDollars, zoomedModuleDollars, sPanelDollarParams);
  }
}

public void unHMI_get_sDpPanel_sName(string fileName, string &sDpPanel, string &sName) {
  dyn_string exceptionInfo;
  if (fileName == "") {
    return;
  }

  sDpPanel=unPanel_convertPanelFileNameToDp(fileName);

  if(!dpExists(sDpPanel)) {
  	unPanel_create(fileName, sDpPanel, exceptionInfo);
	}

  unGenericDpFunctions_getContextPanel(sDpPanel, sName, exceptionInfo);
  return;
}

public void unHMI_updateGraphicVariables(string moduleName, string &dollarParams, string &s_moduleName) {
  int pos = strpos(moduleName, UNHMI_ZOOMED_MODULE_PREFIX);
  int moduleNumber = substr(moduleName, pos + strlen(UNHMI_ZOOMED_MODULE_PREFIX), 1);

  int splitterPos = strpos(moduleName, UNHMI_SPLITTER_FLAG);
  if (splitterPos > 0) {
    dollarParams += ";$"+ UNHMI_SPLITTER_FLAG + ":true";
  }
  dollarParams += ";$number:" + moduleNumber;
  dollarParams  = strltrim(dollarParams, ";");
  s_moduleName = UNHMI_COMMAND_MODULE_NAME + moduleNumber;
}
